/// PMR2415 - Microprocessadores em Automao e Robtica
///
/// Programa para testes iniciais do hardware.
///
/// Esse programa testa os seguintes itens:
///   interrupo pelo Timer 0
///   porta de sada para LED
///   canal serial RS232
///   comunicao com display de LCD do rob
///   comunicao com LED RGB do rob
///   som do rob
///   leitura de nivel de bateria
///   leitura de sensor de proximidade com converso A/D
///   leitura de sensor de linha
///   acionamento de motor com 50% de PWM por Timer 1
///
/// Autor: Jun Okamoto Jr.
/// Data: 12/02/2016
///
/// ATENO: antes de copiar qualquer parte desse cdigo para uso prprio,
/// o aluno deve entender o que o cdigo faz. Caso o aluno seja questionado
/// oralmente e no saiba responder, sua avaliao ser prejudicada.
///

// Gerado automaticamento por "Window > PIC Memory Views > Configuration Bits"
// A partir daqui --------------->
#include <xc.h>

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG1
#pragma config FOSC = EC        // Oscillator Selection bits (EC: I/O function on RA6/OSC2/CLKOUT pin, CLKIN on RA7/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled and can be enabled by SWDTEN bit of the WDTCON register)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = ON       // RE3/MCLR pin function select bit (RE3/MCLR pin function is MCLR)
#pragma config CP = OFF         // Code Protection bit (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)
#pragma config BOREN =OFF       // Brown Out Reset Selection bits (BOR disabled)
#pragma config IESO = ON        // Internal External Switchover bit (Internal/External Switchover mode is enabled)
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enabled bit (Fail-Safe Clock Monitor is enabled)
#pragma config LVP = OFF        // Low Voltage Programming Enable bit (RB3 pin has digital I/O, HV on MCLR must be used for programming)

// CONFIG2
#pragma config BOR4V = BOR21V   // Brown-out Reset Selection bit (Brown-out Reset set to 4.0V)
#pragma config WRT = OFF        // Flash Program Memory Self Write Enable bits (Write protection off)
// <--------------- At aqui

// Includes do projeto
#include <stdio.h>
#include "always.h"
#include "delay.h"
#include "lcd8x2.h"
#include "serial.h"
#include "pwm.h"
#include "sensor.h"
#include "key.h"
#include "battery.h"
#include "spi.h"
#include "led_rgb.h"
#include "compass.h"
#include "robolab2.h"

// Definies
#define VERSION "2.0"

// Timer 0
#define TMR0_PRESCALER 7                     ///< divide por 256
#define POLLING_PERIOD 98                    ///< 5,0176 ms
#define TMR0_SETTING (0xff - POLLING_PERIOD) ///< valor inicial da contagem do Timer 0

// Sadas
#define LED RB5                              ///< bit de sada para o LED
#define BUZZER RB7                           ///< bit para buzzer

// Sadas para simular PWM, no deve ser usado com o PWM real
#define PWM1 RC2  // somente para teste de hw
#define PWM2 RC1  // somente para teste de hw

// variveis globais vo aqui se existirem
// todas as variveis globais usadas em interrupo precisam ser "volatile"

// Ateno: essta rotina no deve ser chamada em nenhum lugar do seu programa.
// Ela  chamada somente na ocorrencia de uma interrupo.
// No caso deste programa, o timer 0  programado para gerar um pedido de
// interrupo a cada ~5 ms.
void interrupt isr(void) {      // Rotina geral de tratamento de interrupo
  // variveis locais declaradas static mantm o valor
  static int tick;              // contador de vezes que o Timer 0 interrompe
  
  // Timer 0
  // Interrompe a cada 5 ms aproximadamente.
  // Faz pisca o LED a cada 1 seg aproximadamente.
  // Controla o tempo de debounce da chave em conjunto com o I-O-C do PORT B
  if (T0IE && T0IF) {    // se for interrupo do Timer 0
    
    // inverte o LED a cada segundo
    if (++tick >= 200) { // se interromper 200x  1 s
      tick = 0;          // ento zera o contador
      LED = ~LED;        // inverte LED      
    } // fim - contagem de vezes que ocorre interrupo
        
    // Debounce da chave. Deve ser includo na interrupp peridica de Timer.
    // So 2 ciclos para debounce entre 5 e 10 ms para interrupo de ~5ms. 
    // Deve ser alterado de acordo com o tempo dado pela interrupo do Timer.
    // Por exemplo, se a interrupo for a cada 1 ms, o parmetro deve ser 10
    // o que resulta num debounce de 9 a 10 ms.
    key_debounce(2); // 2  o nmero de ciclos para dar 5 a 10 ms
    
    TMR0 = TMR0_SETTING;       // recarrega contagem no Timer 0
    T0IF = 0;                  // limpa interrupo
  } // fim - tratamento do Timer 0
  
  // Interrupt-on-change do PORT B
  // O tempo de debounce  controlado pelo Timer 0. A mudana de estado da chave
  // dispara uma contagem de tempo pelo Timer 0 de 5 a 10 ms. Toda a vez que 
  // houver mudana de estado da chave a contagem de tempo de debounce  
  // zerada.
  if (RBIE && RBIF) { // se for mudana estado do Port B
    char portB = PORTB; // faz a leitura do Port B, isso reseta a condio de interrupo junto com RBIF = 0
    key_read(portB);    // faz a leitura da chave
    RBIF = 0;           // reseta o flag de interrupo para poder receber outra interrupo
  } // fim - tratamento I-O-C PORT B
  
  // Timer 1
  // Controla as sadas de PWM para os motores. Essa abordagem s  utilizada
  // para testes do hardware. Na Atividade 6 os alunos devero escrever as 
  // prprias rotinas de PWM e no dever ser utilizado o Timer 1 para tanto.
  if (TMR1IE && TMR1IF) {  // se for interrupo de Timer 1
    PWM1 = ~PWM1;  // inverte a sada do PWM1
    PWM2 = ~PWM2;  // inverte a sada do PWM2
    TMR1 = 0xffff - 1000; // recarrega a constante do Timer 1
    TMR1IF = 0;  // limpa o flag de interrupo
  } // fim - Interrupco do Timer 1
  
  // Outras interrupes aqui
  
} // fim - Tratamento de todas as interrues

/// Toca um beep
/// Para usar o BUZZER o SENSOR POWER deve estar ligado.
void beep() {
  BUZZER = ON;        // liga buzzer
  delay_big_ms(200);  // por 0,2s
  BUZZER = OFF;       // desliga buzzer
}

/// Mostra mensagem inicial no LCD
void welcome_message() {
  char sVar[9];          // string de 8 caracateres
  lcd_goto(0);           // vai para incio da 1a. linha
  lcd_puts("Teste Hw");  // mostra string no LCD para saber que est funcionando
  delay_ms(1);           // d um tempo para o display mostrar a mensagem       
  sprintf(sVar, "v%s    ", VERSION); // armazena verso em forma de texto
  lcd_goto(64);          // vai para o incio da 2a. linha
  lcd_puts(sVar);        // coloca no. da verso  
}

/// Programa Principal
void main (void) {
  
  // variveis locais
  char sVar[9];     // string auxiliar para 8 caracteres
  int countKey = 0; // contador do nmero de vezes que a chave foi pressionada
  static bit lineSensorState;   // indica leitura contnua do sensor de linha
  static bit nearSensorState;   // indica leitura contnua do sensor de proximidade
  static bit batteryLevelState; // indica leitura contnua do nvel de bateria
  static bit compassState;      // indica leitura contnua da bussola
  int nearSensorData;           // valor do sensor de proximidade
  union chartype lineSensor;    // valor do sensor de linha
  char serialIn = 255;          // caracter recebido pelo canal serial
  char keyIn = FALSE;           // tecla pressionada, TRUE = sim
  
  //
  // configuraes dos perifricos
  //
  
  // Timer 0
  // o Timer 0  utilizado para interrupo peridica a cada 5 ms aproximadamente
  OPTION_REGbits.T0CS = 0; // usa clock interno FOSC/4
  OPTION_REGbits.PSA = 0;  // Prescaler  para Timer 0 e no para WDT
  OPTION_REGbits.PS = TMR0_PRESCALER;  // ajusta Prescaler do Timer 0
  
  // Port B
  TRISB5 = 0;  // RB5  sada para LED
  ANS13 = 0;   // RB5/AN13  digital
  TRISB7 = 0;  // RB7  sada para BUZZER
  
  // Controle de interrupes
  T0IE = 1;   // habilita interrupo do Timer 0
  TMR1IE = 0; // inicialmente interrupo est desligada para Timer 1
  PEIE = 1;   // habilita interrupes de perifricos para poder usar Timer 1
  GIE = 1;    // habilita todas as interrupes

  //
  // Inicializaes
  //
  
  // Inicializa o rob
  robolab2_init(); // inicializa todos os perifricos do rob: spi, battery,
                   // lcd, led rgb, compass

  // Inicializaes da placa local
  key_init();     // inicializa chave
  pwm_init();     // inicializa PWM
  serial_init();  // inicializa canal de comunicao serial
  sensor_init();  // inicializa sensores de proximidade de de linha

  
  //
  // Configuraes iniciais
  //
  
  // Nvel da bateria
  batteryLevelState = OFF; // leitura contnua do nvel da bateria desligado
  
  // Sensores
  sensor_power(ON);      // liga energia dos sensores, demora 40 ms para ligar o sensor de proximidade
  lineSensorState = OFF; // leitura contnua de sensor de linha desligada
  nearSensorState = OFF; // leitura contnua do sensor de proximidade desligada
  
  // Bussola
  compassState = OFF;    // leitura contnua de bussola desligada
  
  // LCD
  delay_s(4);       // espera para poder ler mensagem de reset
  lcd_clear();      // limpa LCD, no deve ser usado dentro de loops pois demora muito
  lcd_show_cursor(OFF);  // desliga cursor do LCD

  // mensagem inicial no LCD
  welcome_message(); // coloca mensagem inicial no LCD 
  beep();       // toca beep para sinalizar que est pronto
  putch('\n');  // nova linha no terminal
  putch('\r');  // incio da linha no terminal
  
  //
  // Loop principal
  //
  while (1) {  // para sempre
    
    serialIn = chkchr();   // captura buffer do canal serial
    keyIn = key_pressed(); // armazena status da chave
    
    // Trata chave pressionada ou caracter recebido pela serial
    if (keyIn == TRUE || serialIn != 255) { // se detectar que a chave foi pressionada ou chegada de caracter pela serial
      
      beep(); // toca um beep
      
      if (keyIn) {  // se for chave pressionada
        countKey++; // incrementa contador de chave pressionada
      } else if (serialIn >= '1' && serialIn <= '8') { // se for caracter recebido pela serial
        countKey = serialIn - '0'; // o valor recebido corresponde ao no. de vezes da chave pressionada
      } else countKey = 8;  // seno  fim de testes
      
      putch(countKey + '0');  // envia o valor pelo serial + '0' corrige valor para ASCII
      putch('-');             // coloca um hifem para separar valores
      sprintf(sVar, "%d       ", countKey); // gera string para display
      led_rgb_set_color(countKey); // coloca cor correspondente  contagem
      lcd_goto(0); // coloca o cursor na 6a. posio da linha de cima do LCD
      lcd_puts(sVar); // mostra contagem no LCD
      delay_ms(1);  // espera um pouco para dar tempo ao LCD
     
      switch (countKey) { // testa item de acordo com a quantidade de vezes que a chave  pressionada

        case 1: // bateria
          batteryLevelState = ON;  // liga leitura continua de nvel de bateria
          nearSensorState = OFF;   // desliga leitura contnua do sensor de proximidade
          lineSensorState = OFF;   // desliga leitura contnua do sensor de linha
          compassState = OFF;      // desliga leitura contnua da bussola
          TMR1IE = 0; // desliga PWM
          PWM1 = 0;   // pra motor 1
          PWM2 = 0;   // pra motor 2
        break;
          
        case 2: // sensor de proximidade
          batteryLevelState = OFF; // desliga leitura contnua de nvel de bateria
          nearSensorState = ON;    // liga leitura contnua do sensor de proximidade
          lineSensorState = OFF;   // desliga leitura contnua do sensor de linha
          compassState = OFF;      // desliga leitura contnua da bussola
          TMR1IE = 0; // desliga PWM
          PWM1 = 0;   // pra motor 1
          PWM2 = 0;   // pra motor 2
          break;
          
        case 3: // sensor de linha
          batteryLevelState = OFF; // desliga leitura continua de nvel de bateria
          nearSensorState = OFF;   // desliga leitura contnua do sensor de proximidade
          lineSensorState = ON;    // liga leitura contnua do sensor de linha
          compassState = OFF;      // desliga leitura contnua da bussola
          TMR1IE = 0; // desliga PWM
          PWM1 = 0;   // pra motor 1
          PWM2 = 0;   // pra motor 2
          break;
          
        case 4: // bussola
          batteryLevelState = OFF; // desliga leitura continua de nvel de bateria
          nearSensorState = OFF;   // desliga leitura contnua do sensor de proximidade
          lineSensorState = OFF;   // desliga leitura contnua do sensor de linha
          compassState = ON;       // liga leitura contnua da bussola
          TMR1IE = 0; // desliga PWM
          PWM1 = 0;   // pra motor 1
          PWM2 = 0;   // pra motor 2
          break;
          
        case 5: // motores para frente 
          batteryLevelState = OFF; // desliga leitura continua de nvel de bateria
          nearSensorState = OFF;   // desliga leitura contnua do sensor de proximidade
          lineSensorState = OFF;   // desliga leitura contnua do sensor de linha
          compassState = OFF;      // desliga leitura contnua de bussola
          pwm_direction(0);// direo para frente
          delay_ms(50);    // tem que esperar antes de religar motores pois o pico de corrente reseta o PIC interno
          TMR1IE = 1;      // liga PWM
          lcd_goto(64);    // vai para linha 2, coluna 1 do LCD
          lcd_puts("Move \x6 \x6"); // mostra no LCD
          break;
          
        case 6: // motores cw
          batteryLevelState = OFF; // desliga leitura continua de nvel de bateria
          nearSensorState = OFF;   // desliga leitura contnua do sensor de proximidade
          lineSensorState = OFF;   // desliga leitura contnua do sensor de linha
          compassState = OFF;      // desliga leitura contnua de bussola
          TMR1IE = 0; // pra PWM
          PWM1 = 0;   // pra motor 1
          PWM2 = 0;   // pra motor 2
          pwm_direction(3); // gira cw
          delay_ms(50);  // tem que esperar antes de religar motores pois o pico de corrente reseta o PIC interno
          TMR1IE = 1;    // liga PWM
          lcd_goto(64);    // vai para linha 2, coluna 1 do LCD
          lcd_puts("Move \x6 \x7"); // mostra no LCD
          break;
          
        case 7: // motores ccw
          batteryLevelState = OFF; // desliga leitura continua de nvel de bateria
          nearSensorState = OFF;   // desliga leitura contnua do sensor de proximidade
          lineSensorState = OFF;   // desliga leitura contnua do sensor de linha
          compassState = OFF;      // desliga leitura contnua de bussola
          TMR1IE = 0; // desliga PWM
          PWM1 = 0;   // pra motor 1
          PWM2 = 0;   // pra motor 2
          pwm_direction(2); // muda direo
          delay_ms(50);  // tem que esperar antes de religar motores pois o pico de corrente reseta o PIC interno
          TMR1IE = 1;    // liga PWM
          lcd_goto(64);  // vai para linha 2, coluna 1 do LCD
          lcd_puts("Move \x7 \x6"); // mostra no LCD
          break;
          
        default: // pra motores e retorna para condio inical
          batteryLevelState = OFF; // desliga leitura continua de nvel de bateria
          nearSensorState = OFF;   // desliga leitura contnua do sensor de proximidade
          lineSensorState = OFF;   // desliga leitura contnua do sensor de linha
          compassState = OFF;      // desliga leitura contnua de bussola
          TMR1IE = 0; // pra PWM com Timer 1
          PWM1 = 0;   // pra motor 1
          PWM2 = 0;   // pra motor 2
          pwm_direction(0);  // direo em frente
          countKey = 0;      // reinicializa contagem de vezes de tecla pressionada
          welcome_message(); // recoloca monsagem inicial 
      } // fim - testes
    
    } // fim - chave pressionada ou caracter recebido
    
    // trata medidas continuas
    if (batteryLevelState) { // nvel de bateria
      sprintf(sVar, "Bat: %d", battery_level()); // monta string com valor
      lcd_goto(64);   // vai para linha 2, coluna 1 do LCD
      lcd_puts(sVar); // mostra no LCD
    }
  
    if (nearSensorState) { // sensor de proximidade
      nearSensorData = sensorNear_read(); // l valor do sensor de proximidade
      sprintf(sVar, "Prox:%03d", nearSensorData); // monta string com valor
      lcd_goto(64);   // vai para linha 2, coluna 1 do LCD
      lcd_puts(sVar); // mostra no LCD
      
    }
    
    if (lineSensorState) { // sensor de linha
      lineSensor.byte = sensorLine_read();// l valor do sensor de linha
      sprintf(sVar, "L: %d-%d-%d", lineSensor.part.bit0, lineSensor.part.bit1, lineSensor.part.bit2);
      lcd_goto(64);   // vai para linha 2, coluna 1 do LCD
      lcd_puts(sVar); // mostra no LCD
    }
    
    if (compassState) { // bussola
      uint8_t compass_id = compass_get_id(); // pega o identificado da bssola
      // identificador vai ser 0x3d se estiver funcionando, 0 em caso contrrio
      sprintf(sVar, "Head:%03x", compass_id);  // monta string com valor
      lcd_goto(64);   // vai para linha 2, coluna 1 do LCD
      lcd_puts(sVar); // mostra no LCD
    }
    
    delay_ms(100);    // espera para o LCD no variar muito rpido
    
  } // fim - loop principal
} // fim - main
